==> jmb.tools-0.3.1.tar.gz: jmb.tools-0.3.1/jmb/tools/conf/nginx.py <==
from __future__ import absolute_import
## https://github.com/fatiherikli/nginxparser/blob/master/nginxparser.py

import re
import string
import socket

import click
import DNS
from DNS import Base
from jmb.tools.conf import read
from jmb.tools.test import domains

from pyparsing import (
    Literal, White, Word, alphanums, CharsNotIn, Forward, Group, SkipTo,
    Optional, OneOrMore, ZeroOrMore, pythonStyleComment)


class NginxParser(object):
    """
    A class that parses nginx configuration with pyparsing
    """

    # constants
    left_bracket = Literal("{").suppress()
    right_bracket = Literal("}").suppress()
    semicolon = Literal(";").suppress()
    space = White().suppress()
    key = Word(alphanums + "_/")
    value = CharsNotIn("{};")
    value2 = CharsNotIn(";")
    location = CharsNotIn("{};," + string.whitespace)
    ifword = Literal("if")
    setword = Literal("set")
    # modifier for location uri [ = | ~ | ~* | ^~ ]
    modifier = Literal("=") | Literal("~*") | Literal("~") | Literal("^~")

    # rules
    assignment = (key + Optional(space + value) + semicolon)
    setblock = (setword + OneOrMore(space + value2) + semicolon)
    block = Forward()
    ifblock = Forward()
    subblock = Forward()

    ifblock << (
        ifword
        + SkipTo('{')
        + left_bracket
        + subblock
        + right_bracket)

    subblock << ZeroOrMore(
        Group(assignment) | block | ifblock | setblock
    )

    block << Group(
        Group(key + Optional(space + modifier) + Optional(space + location))
        + left_bracket
        + Group(subblock)
        + right_bracket
    )

    script = OneOrMore(Group(assignment) | block).ignore(pythonStyleComment)

    def __init__(self, source):
        self.source = source

    def parse(self):
        """
        Returns the parsed tree.
        """
        return self.script.parseString(self.source)

    def as_list(self):
        """
        Returns the list of tree.
        """
        return self.parse().asList()


class NginxDumper(object):
    """
    A class that dumps nginx configuration from the provided tree.
    """
    def __init__(self, blocks, indentation=4):
        self.blocks = blocks
        self.indentation = indentation

    def __iter__(self, blocks=None, current_indent=0, spacer=' '):
        """
        Iterates the dumped nginx content.
        """
        blocks = blocks or self.blocks
        for key, values in blocks:
            if current_indent:
                yield spacer
            indentation = spacer * current_indent
            if isinstance(key, list):
                yield indentation + spacer.join(key) + ' {'
                for parameter in values:
                    if isinstance(parameter[0], list):
                        dumped = self.__iter__(
                            [parameter],
                            current_indent + self.indentation)
                        for line in dumped:
                            yield line
                    else:
                        dumped = spacer.join(parameter) + ';'
                        yield spacer * (
                            current_indent + self.indentation) + dumped

                yield indentation + '}'
            else:
                yield spacer * current_indent + key + spacer + values + ';'

    def as_string(self):
        return '\n'.join(self)

    def to_file(self, out):
        for line in self:
            out.write(line+"\n")
        out.close()
        return out


# Shortcut functions to respect Python's serialization interface
# (like pyyaml, picker or json)

def loads(source):
    return NginxParser(source).as_list()


def load(_file):
    return loads(_file.read())


def dumps(blocks, indentation=4):
    return NginxDumper(blocks, indentation).as_string()


def dump(blocks, _file, indentation=4):
    return NginxDumper(blocks, indentation).to_file(_file)

## Added by sandro

class ParseConf(object):

    def __init__(self, filename):
        resp = load(open(filename))
        self.stanzas = [Stanza(x) for x in resp]
        self.servers = [Server(stanza.conf) for stanza in self.stanzas if stanza.is_server()]

class Stanza(object):

    def __init__(self, conf):
        self.conf = conf
        self._type = conf[0][0]

    def is_server(self):
        return self._type == 'server'

class Server(Stanza):

    @property
    def names(self):
        names = []
        for x in self.conf[1]:
            if len(x) > 1 and x[0] == 'server_name':
                names += x[1].split()
        return names

    @property
    def ports(self):
        ports = {'http': [], 'https': []}
        for x in self.conf[1]:
            if len(x) > 1 and x[0] == 'listen':
                value = x[1]
                ## FIXME: you can have https declared  out of listen  directive!!!
                if 'ssl' in value:
                    ports['https'] += [x[1].split()[0]]
                else:
                    ports['http'] += [x[1].split()[0]]
        return ports

    def test_response(self, name, ip, nameserver=None, verbose=False, follow_redirects=False):
        "Check the response, just for srvs that point to ip"

        dns_result = self.test_resolve(name, ip, nameserver)
        if dns_result is True:
            for port in self.ports['http']:
                url = "http://{}/".format(name)
                click.echo('    ', nl=False)
                domains.test_url(url, verbose=False, follow_redirects=False)
            for port in self.ports['https']:
                url = "https://{}/".format(name)
                click.echo('    ', nl=False)
                domains.test_url(url, verbose=False, follow_redirects=False)
        else:
            click.secho("    {} doesn't point here ({})".format(name, dns_result), fg='red')

    def test_all_name_responses(self, ip, nameserver=None, verbose=False, follow_redirects=False):

        for name in self.names:
            if name == '_':
                continue
            self.test_response(name, ip, nameserver=None, verbose=False, follow_redirects=False)

    def test_resolve(self, name, ip, nameserver=None):
        "Check if ip matches the name, return True or the ip it would resolve to"

        try:
            dns_result = resolve_ip(name, rdtype='A', nameservers=nameserver)
        except Base.ServerError:
            return None
        return ip == dns_result[-1] and True or dns_result

    def test_name_class(self, name, nameserver=None):
        """Check if name is CNAME or A"""

        try:
            ret = resolve_ip(name, 'A', nameservers=nameserver)
        except Base.ServerError:
            return ''
        ## Disreguarding possible multiple ip, the goal is just to tel if it's CNAME of A
        return ret[0]

    def test_all_name_classes(self, ip, nameserver=None):
        """Check all names for classes"""

        for name in self.names:
            res = self.test_name_class(name, nameserver=nameserver)
            click.secho("   {}... ".format(name), nl=False)
            if re.search('[a-z]', res):
                ## CNAME
                click.secho(res, fg='yellow')
            else:
                click.secho(res, fg='green')

    def test_name_classes(self, name):
        """Check all names in server config for they're classes"""
        ip = ip or read.get_ip()
        for name in self.names:
            result = self.test_resolve(name, ip)
            click.secho("    {}... ".format(name), nl=False)
            if result is True:
                click.secho('OK', fg='green')
            else:
                click.secho('KO {}'.format(result or '-'), fg='red')



    def test_all_names(self, ip=None, nameserver=None):
        """Check if all server_names point here

        :arg ip: the ip this machine should point (default what is found in get_ip())
        """
        ip = ip or read.get_ip()
        for name in self.names:
            result = self.test_resolve(name, ip, nameserver=nameserver)
            click.secho('    {}... '.format(name), nl=False)
            if result is True:
                click.secho('OK', fg='green')
            else:
                click.secho('KO {}'.format(result or '-'), fg='red')

def resolve_ip(name, rdtype='A', nameservers='8.8.8.8'):

    if nameservers:
        if isinstance(nameservers, basestring):
            nameservers = [nameservers]
    DNS.defaults['servers'] = nameservers
    return DNS.dnslookup(name, rdtype)

==> jmb.tools-0.3.2.tar.gz: jmb.tools-0.3.2/jmb/tools/conf/nginx.py <==
from __future__ import absolute_import
## https://github.com/fatiherikli/nginxparser/blob/master/nginxparser.py

import re
import string
import socket

import click
import DNS
from DNS import Base
from jmb.tools.conf import read
from jmb.tools.test import domains

from pyparsing import (
    Literal, White, Word, alphanums, CharsNotIn, Forward, Group, SkipTo,
    Optional, OneOrMore, ZeroOrMore, pythonStyleComment)


class NginxParser(object):
    """
    A class that parses nginx configuration with pyparsing
    """

    # constants
    left_bracket = Literal("{").suppress()
    right_bracket = Literal("}").suppress()
    semicolon = Literal(";").suppress()
    space = White().suppress()
    key = Word(alphanums + "_/")
    value = CharsNotIn("{};")
    value2 = CharsNotIn(";")
    location = CharsNotIn("{};," + string.whitespace)
    ifword = Literal("if")
    setword = Literal("set")
    # modifier for location uri [ = | ~ | ~* | ^~ ]
    modifier = Literal("=") | Literal("~*") | Literal("~") | Literal("^~")

    # rules
    assignment = (key + Optional(space + value) + semicolon)
    setblock = (setword + OneOrMore(space + value2) + semicolon)
    block = Forward()
    ifblock = Forward()
    subblock = Forward()

    ifblock << (
        ifword
        + SkipTo('{')
        + left_bracket
        + subblock
        + right_bracket)

    subblock << ZeroOrMore(
        Group(assignment) | block | ifblock | setblock
    )

    block << Group(
        Group(key + Optional(space + modifier) + Optional(space + location))
        + left_bracket
        + Group(subblock)
        + right_bracket
    )

    script = OneOrMore(Group(assignment) | block).ignore(pythonStyleComment)

    def __init__(self, source):
        self.source = source

    def parse(self):
        """
        Returns the parsed tree.
        """
        return self.script.parseString(self.source)

    def as_list(self):
        """
        Returns the list of tree.
        """
        return self.parse().asList()


class NginxDumper(object):
    """
    A class that dumps nginx configuration from the provided tree.
    """
    def __init__(self, blocks, indentation=4):
        self.blocks = blocks
        self.indentation = indentation

    def __iter__(self, blocks=None, current_indent=0, spacer=' '):
        """
        Iterates the dumped nginx content.
        """
        blocks = blocks or self.blocks
        for key, values in blocks:
            if current_indent:
                yield spacer
            indentation = spacer * current_indent
            if isinstance(key, list):
                yield indentation + spacer.join(key) + ' {'
                for parameter in values:
                    if isinstance(parameter[0], list):
                        dumped = self.__iter__(
                            [parameter],
                            current_indent + self.indentation)
                        for line in dumped:
                            yield line
                    else:
                        dumped = spacer.join(parameter) + ';'
                        yield spacer * (
                            current_indent + self.indentation) + dumped

                yield indentation + '}'
            else:
                yield spacer * current_indent + key + spacer + values + ';'

    def as_string(self):
        return '\n'.join(self)

    def to_file(self, out):
        for line in self:
            out.write(line+"\n")
        out.close()
        return out


# Shortcut functions to respect Python's serialization interface
# (like pyyaml, picker or json)

def loads(source):
    return NginxParser(source).as_list()


def load(_file):
    return loads(_file.read())


def dumps(blocks, indentation=4):
    return NginxDumper(blocks, indentation).as_string()


def dump(blocks, _file, indentation=4):
    return NginxDumper(blocks, indentation).to_file(_file)

## Added by sandro

class ParseConf(object):

    def __init__(self, filename):
        resp = load(open(filename))
        self.stanzas = [Stanza(x) for x in resp]
        self.servers = [Server(stanza.conf) for stanza in self.stanzas if stanza.is_server()]

class Stanza(object):

    def __init__(self, conf):
        self.conf = conf
        self._type = conf[0][0]

    def is_server(self):
        return self._type == 'server'

class Server(Stanza):

    @property
    def names(self):
        names = []
        for x in self.conf[1]:
            if len(x) > 1 and x[0] == 'server_name':
                names += x[1].split()
        return names

    @property
    def ports(self):
        try:
            return self._ports
        except AttributeError:
            pass
        ports = {'http': [], 'https': []}
        for x in self.conf[1]:
            if len(x) > 1 and x[0] == 'listen':
                value = x[1]
                ## FIXME: you can have https declared  out of listen  directive!!!
                if 'ssl' in value:
                    ports['https'] += [x[1].split()[0]]
                else:
                    ports['http'] += [x[1].split()[0]]
        self._ports = ports
        return ports

    def test_response(self, name, ip, nameserver=None, verbose=False, follow_redirects=False, ssl=False):
        """Check the response, just for srvs that point to ip

        :arg name: the server_name we're testing
        :arg ip: ower ip, unless dns lookup points here, we don't proceed
        :arg nameserver: the nameserver we want to use (default 8.8.8.8)
        :arg verbose: be verbose...
        :arg follow_redirects: don't just show the location in case of redirects
        :arg ssl: If True, test the ssl even it it's not configured in nginx. Just to put in evidence a misconfiguration

        """
        if ssl:
            if not '443' in self.ports['https']:
                self._ports['https'] += ['443']

        dns_result = self.test_resolve(name, ip, nameserver)
        if dns_result is True:
            for port in self.ports['http']:
                url = "http://{}/".format(name)
                click.echo('    ', nl=False)
                domains.test_url(url, verbose=False, follow_redirects=False)
            for port in self.ports['https']:
                url = "https://{}/".format(name)
                click.echo('    ', nl=False)
                domains.test_url(url, verbose=False, follow_redirects=False)
        else:
            click.secho("    {} doesn't point here ({})".format(name, dns_result), fg='red')

    def test_all_name_responses(self, ip, nameserver=None, verbose=False, follow_redirects=False, ssl=False):
        """Loop on all names and test the response. All parameters handled to ``test_response``"""

        for name in self.names:
            if name == '_':
                continue
            self.test_response(name, ip, nameserver=None, verbose=False, follow_redirects=False, ssl=ssl)

    def test_resolve(self, name, ip, nameserver=None):
        "Check if ip matches the name, return True or the ip it would resolve to"

        try:
            dns_result = resolve_ip(name, rdtype='A', nameservers=nameserver)
        except Base.ServerError:
            return None
        return ip == dns_result[-1] and True or dns_result

    def test_name_class(self, name, nameserver=None):
        """Check if name is CNAME or A"""

        try:
            ret = resolve_ip(name, 'A', nameservers=nameserver)
        except Base.ServerError:
            return ''
        ## Disreguarding possible multiple ip, the goal is just to tel if it's CNAME of A
        return ret[0]

    def test_all_name_classes(self, ip, nameserver=None):
        """Check all names for classes"""

        for name in self.names:
            res = self.test_name_class(name, nameserver=nameserver)
            click.secho("   {}... ".format(name), nl=False)
            if re.search('[a-z]', res):
                ## CNAME
                click.secho(res, fg='yellow')
            else:
                click.secho(res, fg='green')

    def test_name_classes(self, name):
        """Check all names in server config for they're classes"""
        ip = ip or read.get_ip()
        for name in self.names:
            result = self.test_resolve(name, ip)
            click.secho("    {}... ".format(name), nl=False)
            if result is True:
                click.secho('OK', fg='green')
            else:
                click.secho('KO {}'.format(result or '-'), fg='red')



    def test_all_names(self, ip=None, nameserver=None):
        """Check if all server_names point here

        :arg ip: the ip this machine should point (default what is found in get_ip())
        """
        ip = ip or read.get_ip()
        for name in self.names:
            result = self.test_resolve(name, ip, nameserver=nameserver)
            click.secho('    {}... '.format(name), nl=False)
            if result is True:
                click.secho('OK', fg='green')
            else:
                click.secho('KO {}'.format(result or '-'), fg='red')

def resolve_ip(name, rdtype='A', nameservers='8.8.8.8'):

    if nameservers:
        if isinstance(nameservers, basestring):
            nameservers = [nameservers]
    DNS.defaults['servers'] = nameservers
    return DNS.dnslookup(name, rdtype)
